@@ -71,7 +71,7 @@ class create_client():
 
         self.debug = debug
         self.history = HistoryPlugin()
-        self.API_VERSION = "0.0.8"  # TODO - Get the version from versioneer
+        self.API_VERSION = "0.0.11"  # TODO - Get the version from versioneer
 
         service_wsdl = '{}/cxf/OPDMSoapInterface?wsdl'.format(server)
         auth_wsdl = '{}/cxf/OPDMSoapInterface/SoapAuthentication?wsdl'.format(server)
@@ -156,7 +156,7 @@ class create_client():
                         </sm:part>
                     </sm:Query>"""
 
-        GetContentResult = """<?xml version="1.0" encoding="UTF-8"?>
+        GetContentResult_Profile = """<?xml version="1.0" encoding="UTF-8"?>
                                 <sm:GetContent xmlns="http://entsoe.eu/opde/ServiceModel/1/0"
                                                xmlns:sm="http://entsoe.eu/opde/ServiceModel/1/0"
                                                xmlns:opde="http://entsoe.eu/opde/ObjectModel/1/0"
@@ -169,6 +169,19 @@ class create_client():
                                         </sm:part>
                                 </sm:GetContent>"""
 
+        GetContentResult_OPDMObject = """<?xml version="1.0" encoding="UTF-8"?>
+                                <sm:GetContent xmlns="http://entsoe.eu/opde/ServiceModel/1/0"
+                                               xmlns:sm="http://entsoe.eu/opde/ServiceModel/1/0"
+                                               xmlns:opde="http://entsoe.eu/opde/ObjectModel/1/0"
+                                               xmlns:opdm="http://entsoe.eu/opdm/ObjectModel/1/0">
+                                    <sm:part name="identifier" type="opde:ShortMetaData">
+                                    <sm:part name="content-return-mode">{return_mode}</sm:part><!-- PAYLOAD or FILE -->
+                                        <opdm:OPDMObject>
+                                            <opde:Id>{mRID}</opde:Id>
+                                        </opdm:OPDMObject>
+                                        </sm:part>
+                                </sm:GetContent>"""
+
         PublicationsSubscriptionList = """<?xml version="1.0" encoding="UTF-8"?>
                                           <sm:PublicationsSubscriptionList xmlns:sm="http://entsoe.eu/opde/ServiceModel/1/0">
                                                 <sm:part name="listType">AVAILABLE_PUBLICATIONS</sm:part>
@@ -310,10 +323,12 @@ class create_client():
 
         return query_id, result
 
-    def get_content(self, content_id, return_payload=False):
+    def get_content(self, content_id, return_payload=False, object_type="file"):
         """Downloads single file from OPDM Service Provider to OPDM Client local storage,
         to get the file binary as a response of set return_payload to True
 
+        supported object_types = [file, model]
+
         Returns a dictionary with metadata and filename or content, to get the filename or content use:
         result['sm:GetContentResult']['sm:part'][1]['opdm:Profile']['opde:Content']
         """
@@ -324,13 +339,16 @@ class create_client():
 
         logger.debug(f"Return mode: {return_mode}")
 
-        get_content_result = self.Operations.GetContentResult.format(mRID=content_id, return_mode=return_mode)
+        object_types = {"file": self.Operations.GetContentResult_Profile,
+                        "model": self.Operations.GetContentResult_OPDMObject}
+
+        get_content_result = object_types[object_type].format(mRID=content_id, return_mode=return_mode)
 
         result = xmltodict.parse(etree.tostring(self.execute_operation(get_content_result.encode())), xml_attribs=False)
 
         if type(result['sm:GetContentResult']['sm:part']) == list:
             logger.info("File downloaded")
-            logger.debug(result['sm:GetContentResult']['sm:part'][1]['opdm:Profile']['opde:Content'])
+            #logger.debug(result['sm:GetContentResult']['sm:part'][1]['opdm:Profile']['opde:Content'])
 
         else:
             logger.error("File download failed")
@@ -342,7 +360,7 @@ class create_client():
         result = self.execute_operation(self.Operations.PublicationsSubscriptionList.encode())
         return xmltodict.parse(etree.tostring(result), xml_attribs=True)
 
-    def publication_subscribe(self, object_type="BDS", subscription_id="", publication_id="", mode="DIRECT_CONTENT", metadata_dict={}):
+    def publication_subscribe(self, object_type="BDS", subscription_id="", publication_id="", mode="DIRECT_CONTENT", metadata_dict=None):
         """
         Set up subscription for data models. By default sets up subscription for BDS
 
@@ -352,8 +370,13 @@ class create_client():
         mode -> META, DIRECT_CONTENT, FULL
         metadata_dict_example = {'pmd:TSO': 'ELERING', 'pmd:timeHorizon': '1D'}
         """
-        object_types = ["IGM", "CGM", "BDS"]
-        if object_type not in object_types:
+        # Get available publications
+        available_publications = self.publication_list()
+
+
+        object_types = {item['opde:messageType']["@v"].split("-")[-1]:item['opde:publicationID']["@v"] for item in available_publications['sm:PublicationsSubscriptionListResult']['sm:part']['opdm:PublicationsList']['opdm:Publication']}
+
+        if object_type not in object_types.keys():
             logger.warning(f"ObjectType '{object_type}' not supported, supported types are: {object_types}")
             return None
 
@@ -365,13 +388,10 @@ class create_client():
         if subscription_id == "":
             subscription_id = str(uuid.uuid4())
 
-        # Get available publications
-        available_publications = self.publication_list()
-        publications_ids = [item['opde:publicationID']["@v"] for item in available_publications['sm:PublicationsSubscriptionListResult']['sm:part']['opdm:PublicationsList']['opdm:Publication']]
-
         if publication_id == "":
-            publication_id = random.choice([id for id in publications_ids if f"-{object_type}" in id])
+            publication_id = object_types[object_type]
 
+        publications_ids = [item['opde:publicationID']["@v"] for item in available_publications['sm:PublicationsSubscriptionListResult']['sm:part']['opdm:PublicationsList']['opdm:Publication']]
         if publication_id not in publications_ids:
             logger.error(f"Publication '{publication_id}' not supported, supported modes are: {publications_ids}")
             return None
@@ -381,7 +401,7 @@ class create_client():
                                                                             mode=mode,
                                                                             object_type=object_type).encode()
 
-        if metadata_dict != "":
+        if metadata_dict:
             publication_subscribe = add_xml_elements(publication_subscribe, ".//opdm:OPDMObject", metadata_dict)
 
         return xmltodict.parse(etree.tostring(self.execute_operation(publication_subscribe)), xml_attribs=False)
